@@ -1,8 +1,9 @@
-/* A Bison parser, made by GNU Bison 3.2.1.108-19b4e.  */
+/* A Bison parser, made by GNU Bison 3.3.90.  */
 
 /* Bison interface for Yacc-like parsers in C
 
-   Copyright (C) 1984, 1989-1990, 2000-2015, 2018 Free Software Foundation, Inc.
+   Copyright (C) 1984, 1989-1990, 2000-2015, 2018-2019 Free Software Foundation,
+   Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -51,11 +52,11 @@
 extern int gram_debug;
 #endif
 /* "%code requires" blocks.  */
-#line 21 "src/parse-gram.y" /* yacc.c:1917  */
+#line 21 "src/parse-gram.y"
 
   #include "symlist.h"
   #include "symtab.h"
-#line 218 "src/parse-gram.y" /* yacc.c:1917  */
+#line 248 "src/parse-gram.y"
 
   typedef enum
   {
@@ -64,7 +65,7 @@ extern int gram_debug;
     param_parse  = 1 << 1,
     param_both   = param_lex | param_parse
   } param_type;
-#line 660 "src/parse-gram.y" /* yacc.c:1917  */
+#line 691 "src/parse-gram.y"
 
   #include "muscle-tab.h"
   typedef struct
@@ -73,7 +74,7 @@ extern int gram_debug;
     muscle_kind kind;
   } value_type;
 
-#line 77 "src/parse-gram.h" /* yacc.c:1917  */
+#line 78 "src/parse-gram.h"
 
 /* Token type.  */
 #ifndef GRAM_TOKENTYPE
@@ -98,49 +99,50 @@ extern int gram_debug;
     PERCENT_DEFAULT_PREC = 272,
     PERCENT_DEFINE = 273,
     PERCENT_DEFINES = 274,
-    PERCENT_EXPECT = 275,
-    PERCENT_EXPECT_RR = 276,
-    PERCENT_FLAG = 277,
-    PERCENT_FILE_PREFIX = 278,
-    PERCENT_GLR_PARSER = 279,
-    PERCENT_INITIAL_ACTION = 280,
-    PERCENT_LANGUAGE = 281,
-    PERCENT_NAME_PREFIX = 282,
-    PERCENT_NO_DEFAULT_PREC = 283,
-    PERCENT_NO_LINES = 284,
-    PERCENT_NONDETERMINISTIC_PARSER = 285,
-    PERCENT_OUTPUT = 286,
-    PERCENT_REQUIRE = 287,
-    PERCENT_SKELETON = 288,
-    PERCENT_START = 289,
-    PERCENT_TOKEN_TABLE = 290,
-    PERCENT_VERBOSE = 291,
-    PERCENT_YACC = 292,
-    BRACED_CODE = 293,
-    BRACED_PREDICATE = 294,
-    BRACKETED_ID = 295,
-    CHAR = 296,
-    EPILOGUE = 297,
-    EQUAL = 298,
-    ID = 299,
-    ID_COLON = 300,
-    PERCENT_PERCENT = 301,
-    PIPE = 302,
-    PROLOGUE = 303,
-    SEMICOLON = 304,
-    TAG = 305,
-    TAG_ANY = 306,
-    TAG_NONE = 307,
-    INT = 308,
-    PERCENT_PARAM = 309,
-    PERCENT_UNION = 310,
-    PERCENT_EMPTY = 311
+    PERCENT_ERROR_VERBOSE = 275,
+    PERCENT_EXPECT = 276,
+    PERCENT_EXPECT_RR = 277,
+    PERCENT_FLAG = 278,
+    PERCENT_FILE_PREFIX = 279,
+    PERCENT_GLR_PARSER = 280,
+    PERCENT_INITIAL_ACTION = 281,
+    PERCENT_LANGUAGE = 282,
+    PERCENT_NAME_PREFIX = 283,
+    PERCENT_NO_DEFAULT_PREC = 284,
+    PERCENT_NO_LINES = 285,
+    PERCENT_NONDETERMINISTIC_PARSER = 286,
+    PERCENT_OUTPUT = 287,
+    PERCENT_REQUIRE = 288,
+    PERCENT_SKELETON = 289,
+    PERCENT_START = 290,
+    PERCENT_TOKEN_TABLE = 291,
+    PERCENT_VERBOSE = 292,
+    PERCENT_YACC = 293,
+    BRACED_CODE = 294,
+    BRACED_PREDICATE = 295,
+    BRACKETED_ID = 296,
+    CHAR = 297,
+    COLON = 298,
+    EPILOGUE = 299,
+    EQUAL = 300,
+    ID = 301,
+    ID_COLON = 302,
+    PERCENT_PERCENT = 303,
+    PIPE = 304,
+    PROLOGUE = 305,
+    SEMICOLON = 306,
+    TAG = 307,
+    TAG_ANY = 308,
+    TAG_NONE = 309,
+    INT = 310,
+    PERCENT_PARAM = 311,
+    PERCENT_UNION = 312,
+    PERCENT_EMPTY = 313
   };
 #endif
 
 /* Value type.  */
 #if ! defined GRAM_STYPE && ! defined GRAM_STYPE_IS_DECLARED
-
 union GRAM_STYPE
 {
 
@@ -161,9 +163,9 @@ union GRAM_STYPE
   /* "integer"  */
   int INT;
   /* int.opt  */
-  int yytype_78;
+  int yytype_80;
   /* named_ref.opt  */
-  named_ref* yytype_90;
+  named_ref* yytype_92;
   /* "%param"  */
   param_type PERCENT_PARAM;
   /* token_decl  */
@@ -179,7 +181,7 @@ union GRAM_STYPE
   /* string_as_id  */
   symbol* string_as_id;
   /* string_as_id.opt  */
-  symbol* yytype_97;
+  symbol* yytype_99;
   /* generic_symlist  */
   symbol_list* generic_symlist;
   /* generic_symlist_item  */
@@ -189,17 +191,25 @@ union GRAM_STYPE
   /* token_decls  */
   symbol_list* token_decls;
   /* token_decl.1  */
-  symbol_list* yytype_76;
+  symbol_list* yytype_78;
   /* token_decls_for_prec  */
   symbol_list* token_decls_for_prec;
   /* token_decl_for_prec.1  */
-  symbol_list* yytype_80;
+  symbol_list* yytype_82;
   /* symbol_decls  */
   symbol_list* symbol_decls;
   /* symbol_decl.1  */
-  symbol_list* yytype_83;
+  symbol_list* yytype_85;
+  /* "%error-verbose"  */
+  uniqstr PERCENT_ERROR_VERBOSE;
   /* "%<flag>"  */
   uniqstr PERCENT_FLAG;
+  /* "%file-prefix"  */
+  uniqstr PERCENT_FILE_PREFIX;
+  /* "%name-prefix"  */
+  uniqstr PERCENT_NAME_PREFIX;
+  /* "%yacc"  */
+  uniqstr PERCENT_YACC;
   /* "[identifier]"  */
   uniqstr BRACKETED_ID;
   /* "identifier"  */
@@ -209,7 +219,7 @@ union GRAM_STYPE
   /* "<tag>"  */
   uniqstr TAG;
   /* tag.opt  */
-  uniqstr yytype_70;
+  uniqstr yytype_72;
   /* tag  */
   uniqstr tag;
   /* variable  */
@@ -218,9 +228,9 @@ union GRAM_STYPE
   unsigned char CHAR;
   /* value  */
   value_type value;
-#line 222 "src/parse-gram.h" /* yacc.c:1917  */
-};
+#line 232 "src/parse-gram.h"
 
+};
 typedef union GRAM_STYPE GRAM_STYPE;
 # define GRAM_STYPE_IS_TRIVIAL 1
 # define GRAM_STYPE_IS_DECLARED 1
