@@ -1,7 +1,7 @@
 /* Symbol table manager for Bison.
 
-   Copyright (C) 1984, 1989, 2000-2002, 2004-2015 Free Software
-   Foundation, Inc.
+   Copyright (C) 1984, 1989, 2000-2002, 2004-2015, 2018-2019 Free
+   Software Foundation, Inc.
 
    This file is part of Bison, the GNU Compiler Compiler.
 
@@ -19,13 +19,15 @@
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #include <config.h>
+#include "symtab.h"
+
 #include "system.h"
 
+#include <assure.h>
 #include <hash.h>
 
 #include "complain.h"
 #include "gram.h"
-#include "symtab.h"
 
 /*-------------------------------------------------------------------.
 | Symbols sorted by tag.  Allocated by the first invocation of       |
@@ -46,17 +48,40 @@ symbol *accept = NULL;
 symbol *startsymbol = NULL;
 location startsymbol_location;
 
-/*---------------------------.
-| Precedence relation graph. |
-`---------------------------*/
-
+/* Precedence relation graph. */
 static symgraph **prec_nodes;
 
-/*-----------------------------------.
-| Store which associativity is used. |
-`-----------------------------------*/
+/* Store which associativity is used.  */
+static bool *used_assoc = NULL;
 
-bool *used_assoc = NULL;
+bool tag_seen = false;
+
+
+/*--------------------------.
+| Create a new sym_content. |
+`--------------------------*/
+
+static sym_content *
+sym_content_new (symbol *s)
+{
+  sym_content *res = xmalloc (sizeof *res);
+
+  res->symbol = s;
+
+  res->type_name = NULL;
+  for (int i = 0; i < CODE_PROPS_SIZE; ++i)
+    code_props_none_init (&res->props[i]);
+
+  res->number = NUMBER_UNDEFINED;
+  res->prec = 0;
+  res->assoc = undef_assoc;
+  res->user_token_number = USER_NUMBER_UNDEFINED;
+
+  res->class = unknown_sym;
+  res->status = undeclared;
+
+  return res;
+}
 
 /*---------------------------------.
 | Create a new symbol, named TAG.  |
@@ -76,22 +101,10 @@ symbol_new (uniqstr tag, location loc)
 
   res->tag = tag;
   res->location = loc;
-
-  res->type_name = NULL;
-  {
-    int i;
-    for (i = 0; i < CODE_PROPS_SIZE; ++i)
-      code_props_none_init (&res->props[i]);
-  }
-
-  res->number = NUMBER_UNDEFINED;
-  res->prec = 0;
-  res->assoc = undef_assoc;
-  res->user_token_number = USER_NUMBER_UNDEFINED;
-
+  res->location_of_lhs = false;
   res->alias = NULL;
-  res->class = unknown_sym;
-  res->status = undeclared;
+  res->content = sym_content_new (res);
+  res->is_alias = false;
 
   if (nsyms == SYMBOL_NUMBER_MAXIMUM)
     complain (NULL, fatal, _("too many symbols in input grammar (limit is %d)"),
@@ -100,6 +113,31 @@ symbol_new (uniqstr tag, location loc)
   return res;
 }
 
+/*--------------------.
+| Free a sym_content. |
+`--------------------*/
+
+static void
+sym_content_free (sym_content *sym)
+{
+  free (sym);
+}
+
+
+/*---------------------------------------------------------.
+| Free a symbol and its associated content if appropriate. |
+`---------------------------------------------------------*/
+
+static void
+symbol_free (void *ptr)
+{
+  symbol *sym = (symbol *)ptr;
+  if (!sym->is_alias)
+    sym_content_free (sym->content);
+  free (sym);
+
+}
+
 /* If needed, swap first and second so that first has the earliest
    location (according to location_cmp).
 
@@ -145,9 +183,10 @@ code_props_type_string (code_props_type kind)
     case printer:
       return "%printer";
     }
-  assert (0);
+  abort ();
 }
 
+
 /*----------------------------------------.
 | Create a new semantic type, named TAG.  |
 `----------------------------------------*/
@@ -161,11 +200,8 @@ semantic_type_new (uniqstr tag, const location *loc)
   res->tag = tag;
   res->location = loc ? *loc : empty_location;
   res->status = undeclared;
-  {
-    int i;
-    for (i = 0; i < CODE_PROPS_SIZE; ++i)
-      code_props_none_init (&res->props[i]);
-  }
+  for (int i = 0; i < CODE_PROPS_SIZE; ++i)
+    code_props_none_init (&res->props[i]);
 
   return res;
 }
@@ -176,12 +212,12 @@ semantic_type_new (uniqstr tag, const location *loc)
 `-----------------*/
 
 #define SYMBOL_ATTR_PRINT(Attr)                         \
-  if (s->Attr)                                          \
-    fprintf (f, " %s { %s }", #Attr, s->Attr)
+  if (s->content && s->content->Attr)                   \
+    fprintf (f, " %s { %s }", #Attr, s->content->Attr)
 
 #define SYMBOL_CODE_PRINT(Attr)                                         \
-  if (s->props[Attr].code)                                              \
-    fprintf (f, " %s { %s }", #Attr, s->props[Attr].code)
+  if (s->content && s->content->props[Attr].code)                       \
+    fprintf (f, " %s { %s }", #Attr, s->content->props[Attr].code)
 
 void
 symbol_print (symbol const *s, FILE *f)
@@ -228,7 +264,6 @@ is_identifier (uniqstr s)
 uniqstr
 symbol_id_get (symbol const *sym)
 {
-  aver (sym->user_token_number != USER_NUMBER_HAS_STRING_ALIAS);
   if (sym->alias)
     sym = sym->alias;
   return is_identifier (sym->tag) ? sym->tag : 0;
@@ -241,8 +276,8 @@ symbol_id_get (symbol const *sym)
 `------------------------------------------------------------------*/
 
 static void
-symbol_redeclaration (symbol *s, const char *what, location first,
-                      location second)
+complain_symbol_redeclared (symbol *s, const char *what, location first,
+                            location second)
 {
   unsigned i = 0;
   locations_sort (&first, &second);
@@ -254,8 +289,8 @@ symbol_redeclaration (symbol *s, const char *what, location first,
 }
 
 static void
-semantic_type_redeclaration (semantic_type *s, const char *what, location first,
-                             location second)
+complain_semantic_type_redeclared (semantic_type *s, const char *what, location first,
+                                   location second)
 {
   unsigned i = 0;
   locations_sort (&first, &second);
@@ -266,6 +301,29 @@ semantic_type_redeclaration (semantic_type *s, const char *what, location first,
                    _("previous declaration"));
 }
 
+static void
+complain_class_redeclared (symbol *sym, symbol_class class, location second)
+{
+  unsigned i = 0;
+  complain_indent (&second, complaint, &i,
+                   class == token_sym
+                   ? _("symbol %s redeclared as a token")
+                   : _("symbol %s redeclared as a nonterminal"), sym->tag);
+  if (!location_empty (sym->location))
+    {
+      i += SUB_INDENT;
+      complain_indent (&sym->location, complaint, &i,
+                       _("previous definition"));
+    }
+}
+
+
+void
+symbol_location_as_lhs_set (symbol *sym, location loc)
+{
+  if (!sym->location_of_lhs)
+    sym->location = loc;
+}
 
 
 /*-----------------------------------------------------------------.
@@ -278,13 +336,15 @@ symbol_type_set (symbol *sym, uniqstr type_name, location loc)
 {
   if (type_name)
     {
-      if (sym->type_name)
-        symbol_redeclaration (sym, "%type", sym->type_location, loc);
+      tag_seen = true;
+      if (sym->content->type_name)
+        complain_symbol_redeclared (sym, "%type",
+                                    sym->content->type_location, loc);
       else
         {
           uniqstr_assert (type_name);
-          sym->type_name = type_name;
-          sym->type_location = loc;
+          sym->content->type_name = type_name;
+          sym->content->type_location = loc;
         }
     }
 }
@@ -297,12 +357,12 @@ void
 symbol_code_props_set (symbol *sym, code_props_type kind,
                        code_props const *code)
 {
-  if (sym->props[kind].code)
-    symbol_redeclaration (sym, code_props_type_string (kind),
-                          sym->props[kind].location,
-                          code->location);
+  if (sym->content->props[kind].code)
+    complain_symbol_redeclared (sym, code_props_type_string (kind),
+                                sym->content->props[kind].location,
+                                code->location);
   else
-    sym->props[kind] = *code;
+    sym->content->props[kind] = *code;
 }
 
 /*-----------------------------------------------------.
@@ -315,9 +375,9 @@ semantic_type_code_props_set (semantic_type *type,
                               code_props const *code)
 {
   if (type->props[kind].code)
-    semantic_type_redeclaration (type, code_props_type_string (kind),
-                                 type->props[kind].location,
-                                 code->location);
+    complain_semantic_type_redeclared (type, code_props_type_string (kind),
+                                       type->props[kind].location,
+                                       code->location);
   else
     type->props[kind] = *code;
 }
@@ -330,14 +390,14 @@ code_props *
 symbol_code_props_get (symbol *sym, code_props_type kind)
 {
   /* Per-symbol code props.  */
-  if (sym->props[kind].code)
-    return &sym->props[kind];
+  if (sym->content->props[kind].code)
+    return &sym->content->props[kind];
 
   /* Per-type code props.  */
-  if (sym->type_name)
+  if (sym->content->type_name)
     {
       code_props *code =
-        &semantic_type_get (sym->type_name, NULL)->props[kind];
+        &semantic_type_get (sym->content->type_name, NULL)->props[kind];
       if (code->code)
         return code;
     }
@@ -345,8 +405,8 @@ symbol_code_props_get (symbol *sym, code_props_type kind)
   /* Apply default code props's only to user-defined symbols.  */
   if (sym->tag[0] != '$' && sym != errtoken)
     {
-      code_props *code =
-        &semantic_type_get (sym->type_name ? "*" : "", NULL)->props[kind];
+      code_props *code = &semantic_type_get (sym->content->type_name ? "*" : "",
+                                             NULL)->props[kind];
       if (code->code)
         return code;
     }
@@ -363,14 +423,15 @@ symbol_precedence_set (symbol *sym, int prec, assoc a, location loc)
 {
   if (a != undef_assoc)
     {
-      if (sym->prec)
-        symbol_redeclaration (sym, assoc_to_string (a), sym->prec_location,
-                              loc);
+      sym_content *s = sym->content;
+      if (s->prec)
+        complain_symbol_redeclared (sym, assoc_to_string (a),
+                                    s->prec_location, loc);
       else
         {
-          sym->prec = prec;
-          sym->assoc = a;
-          sym->prec_location = loc;
+          s->prec = prec;
+          s->assoc = a;
+          s->prec_location = loc;
         }
     }
 
@@ -386,27 +447,25 @@ symbol_precedence_set (symbol *sym, int prec, assoc a, location loc)
 void
 symbol_class_set (symbol *sym, symbol_class class, location loc, bool declaring)
 {
-  bool warned = false;
-  if (sym->class != unknown_sym && sym->class != class)
+  aver (class != unknown_sym);
+  sym_content *s = sym->content;
+  if (s->class != unknown_sym && s->class != class)
+    complain_class_redeclared (sym, class, loc);
+  else
     {
-      complain (&loc, complaint, _("symbol %s redefined"), sym->tag);
-      /* Don't report both "redefined" and "redeclared".  */
-      warned = true;
-    }
+      if (class == nterm_sym && s->class != nterm_sym)
+        s->number = nvars++;
+      else if (class == token_sym && s->number == NUMBER_UNDEFINED)
+        s->number = ntokens++;
+      s->class = class;
 
-  if (class == nterm_sym && sym->class != nterm_sym)
-    sym->number = nvars++;
-  else if (class == token_sym && sym->number == NUMBER_UNDEFINED)
-    sym->number = ntokens++;
-
-  sym->class = class;
-
-  if (declaring)
-    {
-      if (sym->status == declared && !warned)
-        complain (&loc, Wother, _("symbol %s redeclared"), sym->tag);
-      else
-        sym->status = declared;
+      if (declaring)
+        {
+          if (s->status == declared)
+            complain (&loc, Wother, _("symbol %s redeclared"), sym->tag);
+          else
+            s->status = declared;
+        }
     }
 }
 
@@ -418,27 +477,27 @@ symbol_class_set (symbol *sym, symbol_class class, location loc, bool declaring)
 void
 symbol_user_token_number_set (symbol *sym, int user_token_number, location loc)
 {
-  int *user_token_numberp;
-
-  if (sym->user_token_number != USER_NUMBER_HAS_STRING_ALIAS)
-    user_token_numberp = &sym->user_token_number;
-  else
-    user_token_numberp = &sym->alias->user_token_number;
-  if (*user_token_numberp != USER_NUMBER_UNDEFINED
-      && *user_token_numberp != user_token_number)
+  int *user_token_numberp = &sym->content->user_token_number;
+  if (sym->content->class != token_sym)
+    complain (&loc, complaint,
+              _("nonterminals cannot be given an explicit number"));
+  else if (*user_token_numberp != USER_NUMBER_UNDEFINED
+           && *user_token_numberp != user_token_number)
     complain (&loc, complaint, _("redefining user token number of %s"),
               sym->tag);
-
-  *user_token_numberp = user_token_number;
-  /* User defined $end token? */
-  if (user_token_number == 0)
+  else
     {
-      endtoken = sym;
-      /* It is always mapped to 0, so it was already counted in
-         NTOKENS.  */
-      if (endtoken->number != NUMBER_UNDEFINED)
-        --ntokens;
-      endtoken->number = 0;
+      *user_token_numberp = user_token_number;
+      /* User defined $end token? */
+      if (user_token_number == 0)
+        {
+          endtoken = sym->content->symbol;
+          /* It is always mapped to 0, so it was already counted in
+             NTOKENS.  */
+          if (endtoken->content->number != NUMBER_UNDEFINED)
+            --ntokens;
+          endtoken->content->number = 0;
+        }
     }
 }
 
@@ -451,29 +510,27 @@ symbol_user_token_number_set (symbol *sym, int user_token_number, location loc)
 static inline bool
 symbol_check_defined (symbol *sym)
 {
-  if (sym->class == unknown_sym)
+  sym_content *s = sym->content;
+  if (s->class == unknown_sym)
     {
-      assert (sym->status != declared);
+      assert (s->status != declared);
       complain (&sym->location,
-                sym->status == needed ? complaint : Wother,
+                s->status == needed ? complaint : Wother,
                 _("symbol %s is used, but is not defined as a token"
                   " and has no rules"),
                   sym->tag);
-      sym->class = nterm_sym;
-      sym->number = nvars++;
+      s->class = nterm_sym;
+      s->number = nvars++;
     }
 
-  {
-    int i;
-    for (i = 0; i < 2; ++i)
-      symbol_code_props_get (sym, i)->is_used = true;
-  }
+  for (int i = 0; i < 2; ++i)
+    symbol_code_props_get (sym, i)->is_used = true;
 
   /* Set the semantic type status associated to the current symbol to
      'declared' so that we could check semantic types unnecessary uses. */
-  if (sym->type_name)
+  if (s->type_name)
     {
-      semantic_type *sem_type = semantic_type_get (sym->type_name, NULL);
+      semantic_type *sem_type = semantic_type_get (s->type_name, NULL);
       if (sem_type)
         sem_type->status = declared;
     }
@@ -489,8 +546,7 @@ semantic_type_check_defined (semantic_type *sem_type)
       || !*sem_type->tag
       || STREQ (sem_type->tag, "*"))
     {
-      int i;
-      for (i = 0; i < 2; ++i)
+      for (int i = 0; i < 2; ++i)
         if (sem_type->props[i].kind != CODE_PROPS_NONE
             && ! sem_type->props[i].is_used)
           complain (&sem_type->location, Wother,
@@ -519,10 +575,49 @@ semantic_type_check_defined_processor (void *sem_type,
 }
 
 
+/*-------------------------------------------------------------------.
+| Merge the properties (precedence, associativity, etc.) of SYM, and |
+| its string-named alias STR; check consistency.                     |
+`-------------------------------------------------------------------*/
+
+static void
+symbol_merge_properties (symbol *sym, symbol *str)
+{
+  if (str->content->type_name != sym->content->type_name)
+    {
+      if (str->content->type_name)
+        symbol_type_set (sym,
+                         str->content->type_name, str->content->type_location);
+      else
+        symbol_type_set (str,
+                         sym->content->type_name, sym->content->type_location);
+    }
+
+
+  for (int i = 0; i < CODE_PROPS_SIZE; ++i)
+    if (str->content->props[i].code)
+      symbol_code_props_set (sym, i, &str->content->props[i]);
+    else if (sym->content->props[i].code)
+      symbol_code_props_set (str, i, &sym->content->props[i]);
+
+  if (sym->content->prec || str->content->prec)
+    {
+      if (str->content->prec)
+        symbol_precedence_set (sym, str->content->prec, str->content->assoc,
+                               str->content->prec_location);
+      else
+        symbol_precedence_set (str, sym->content->prec, sym->content->assoc,
+                               sym->content->prec_location);
+    }
+}
+
 void
 symbol_make_alias (symbol *sym, symbol *str, location loc)
 {
-  if (str->alias)
+  if (sym->content->class != token_sym)
+    complain (&loc, complaint,
+              _("nonterminals cannot be given a string alias"));
+  else if (str->alias)
     complain (&loc, Wother,
               _("symbol %s used more than once as a literal string"), str->tag);
   else if (sym->alias)
@@ -530,71 +625,17 @@ symbol_make_alias (symbol *sym, symbol *str, location loc)
               _("symbol %s given more than one literal string"), sym->tag);
   else
     {
-      str->class = token_sym;
-      str->user_token_number = sym->user_token_number;
-      sym->user_token_number = USER_NUMBER_HAS_STRING_ALIAS;
+      symbol_merge_properties (sym, str);
+      sym_content_free (str->content);
+      str->content = sym->content;
+      str->content->symbol = str;
+      str->is_alias = true;
       str->alias = sym;
       sym->alias = str;
-      str->number = sym->number;
-      symbol_type_set (str, sym->type_name, loc);
     }
 }
 
 
-/*---------------------------------------------------------.
-| Check that THIS, and its alias, have same precedence and |
-| associativity.                                           |
-`---------------------------------------------------------*/
-
-static inline void
-symbol_check_alias_consistency (symbol *this)
-{
-  symbol *sym = this;
-  symbol *str = this->alias;
-
-  /* Check only the symbol in the symbol-string pair.  */
-  if (!(this->alias
-        && this->user_token_number == USER_NUMBER_HAS_STRING_ALIAS))
-    return;
-
-  if (str->type_name != sym->type_name)
-    {
-      if (str->type_name)
-        symbol_type_set (sym, str->type_name, str->type_location);
-      else
-        symbol_type_set (str, sym->type_name, sym->type_location);
-    }
-
-
-  {
-    int i;
-    for (i = 0; i < CODE_PROPS_SIZE; ++i)
-      if (str->props[i].code)
-        symbol_code_props_set (sym, i, &str->props[i]);
-      else if (sym->props[i].code)
-        symbol_code_props_set (str, i, &sym->props[i]);
-  }
-
-  if (sym->prec || str->prec)
-    {
-      if (str->prec)
-        symbol_precedence_set (sym, str->prec, str->assoc,
-                               str->prec_location);
-      else
-        symbol_precedence_set (str, sym->prec, sym->assoc,
-                               sym->prec_location);
-    }
-}
-
-static bool
-symbol_check_alias_consistency_processor (void *this,
-                                          void *null ATTRIBUTE_UNUSED)
-{
-  symbol_check_alias_consistency (this);
-  return true;
-}
-
-
 /*-------------------------------------------------------------------.
 | Assign a symbol number, and write the definition of the token name |
 | into FDEFINES.  Put in SYMBOLS.                                    |
@@ -603,13 +644,11 @@ symbol_check_alias_consistency_processor (void *this,
 static inline bool
 symbol_pack (symbol *this)
 {
-  aver (this->number != NUMBER_UNDEFINED);
-  if (this->class == nterm_sym)
-    this->number += ntokens;
-  else if (this->user_token_number == USER_NUMBER_HAS_STRING_ALIAS)
-    return true;
+  aver (this->content->number != NUMBER_UNDEFINED);
+  if (this->content->class == nterm_sym)
+    this->content->number += ntokens;
 
-  symbols[this->number] = this;
+  symbols[this->content->number] = this->content->symbol;
   return true;
 }
 
@@ -619,9 +658,8 @@ symbol_pack_processor (void *this, void *null ATTRIBUTE_UNUSED)
   return symbol_pack (this);
 }
 
-
 static void
-user_token_number_redeclaration (int num, symbol *first, symbol *second)
+complain_user_token_number_redeclared (int num, symbol *first, symbol *second)
 {
   unsigned i = 0;
   symbols_sort (&first, &second);
@@ -641,18 +679,19 @@ user_token_number_redeclaration (int num, symbol *first, symbol *second)
 static inline bool
 symbol_translation (symbol *this)
 {
-  /* Non-terminal? */
-  if (this->class == token_sym
-      && this->user_token_number != USER_NUMBER_HAS_STRING_ALIAS)
+  /* Nonterminal? */
+  if (this->content->class == token_sym
+      && !this->is_alias)
     {
-      /* A token which translation has already been set? */
-      if (token_translations[this->user_token_number] != undeftoken->number)
-        user_token_number_redeclaration
-          (this->user_token_number,
-           symbols[token_translations[this->user_token_number]],
-           this);
+      /* A token which translation has already been set?*/
+      if (token_translations[this->content->user_token_number]
+          != undeftoken->content->number)
+        complain_user_token_number_redeclared
+          (this->content->user_token_number,
+           symbols[token_translations[this->content->user_token_number]], this);
       else
-        token_translations[this->user_token_number] = this->number;
+        token_translations[this->content->user_token_number]
+          = this->content->number;
     }
 
   return true;
@@ -738,7 +777,7 @@ symbols_new (void)
                                   NULL,
                                   hash_symbol_hasher,
                                   hash_symbol_comparator,
-                                  free);
+                                  symbol_free);
   semantic_type_table = hash_initialize (HT_INITIAL_CAPACITY,
                                          NULL,
                                          hash_semantic_type_hasher,
@@ -756,10 +795,9 @@ symbol *
 symbol_from_uniqstr (const uniqstr key, location loc)
 {
   symbol probe;
-  symbol *entry;
 
   probe.tag = key;
-  entry = hash_lookup (symbol_table, &probe);
+  symbol *entry = hash_lookup (symbol_table, &probe);
 
   if (!entry)
     {
@@ -782,10 +820,9 @@ semantic_type *
 semantic_type_from_uniqstr (const uniqstr key, const location *loc)
 {
   semantic_type probe;
-  semantic_type *entry;
 
   probe.tag = key;
-  entry = hash_lookup (semantic_type_table, &probe);
+  semantic_type *entry = hash_lookup (semantic_type_table, &probe);
 
   if (!entry)
     {
@@ -832,14 +869,12 @@ dummy_symbol_get (location loc)
 {
   /* Incremented for each generated symbol.  */
   static int dummy_count = 0;
-  static char buf[256];
-
-  symbol *sym;
-
-  sprintf (buf, "$@%d", ++dummy_count);
-  sym = symbol_get (buf, loc);
-  sym->class = nterm_sym;
-  sym->number = nvars++;
+  char buf[32];
+  int len = snprintf (buf, sizeof buf, "$@%d", ++dummy_count);
+  assure (len < sizeof buf);
+  symbol *sym = symbol_get (buf, loc);
+  sym->content->class = nterm_sym;
+  sym->content->number = nvars++;
   return sym;
 }
 
@@ -892,11 +927,8 @@ symbols_do (Hash_processor processor, void *processor_data,
       hash_get_entries (table, (void**)*sorted, count);
       qsort (*sorted, count, sizeof **sorted, symbols_cmp_qsort);
     }
-  {
-    size_t i;
-    for (i = 0; i < count; ++i)
-      processor ((*sorted)[i], processor_data);
-  }
+  for (size_t i = 0; i < count; ++i)
+    processor ((*sorted)[i], processor_data);
 }
 
 /*--------------------------------------------------------------.
@@ -922,14 +954,13 @@ static void
 symbols_token_translations_init (void)
 {
   bool num_256_available_p = true;
-  int i;
 
   /* Find the highest user token number, and whether 256, the POSIX
      preferred user token number for the error token, is used.  */
   max_user_token_number = 0;
-  for (i = 0; i < ntokens; ++i)
+  for (int i = 0; i < ntokens; ++i)
     {
-      symbol *this = symbols[i];
+      sym_content *this = symbols[i]->content;
       if (this->user_token_number != USER_NUMBER_UNDEFINED)
         {
           if (this->user_token_number > max_user_token_number)
@@ -941,16 +972,16 @@ symbols_token_translations_init (void)
 
   /* If 256 is not used, assign it to error, to follow POSIX.  */
   if (num_256_available_p
-      && errtoken->user_token_number == USER_NUMBER_UNDEFINED)
-    errtoken->user_token_number = 256;
+      && errtoken->content->user_token_number == USER_NUMBER_UNDEFINED)
+    errtoken->content->user_token_number = 256;
 
   /* Set the missing user numbers. */
   if (max_user_token_number < 256)
     max_user_token_number = 256;
 
-  for (i = 0; i < ntokens; ++i)
+  for (int i = 0; i < ntokens; ++i)
     {
-      symbol *this = symbols[i];
+      sym_content *this = symbols[i]->content;
       if (this->user_token_number == USER_NUMBER_UNDEFINED)
         this->user_token_number = ++max_user_token_number;
       if (this->user_token_number > max_user_token_number)
@@ -962,8 +993,8 @@ symbols_token_translations_init (void)
 
   /* Initialize all entries for literal tokens to the internal token
      number for $undefined, which represents all invalid inputs.  */
-  for (i = 0; i < max_user_token_number + 1; i++)
-    token_translations[i] = undeftoken->number;
+  for (int i = 0; i < max_user_token_number + 1; i++)
+    token_translations[i] = undeftoken->content->number;
   symbols_do (symbol_translation_processor, NULL,
               symbol_table, &symbols_sorted);
 }
@@ -977,18 +1008,13 @@ symbols_token_translations_init (void)
 void
 symbols_pack (void)
 {
-  symbols_do (symbol_check_alias_consistency_processor, NULL,
-              symbol_table, &symbols_sorted);
-
   symbols = xcalloc (nsyms, sizeof *symbols);
   symbols_do (symbol_pack_processor, NULL, symbol_table, &symbols_sorted);
 
   /* Aliases leave empty slots in symbols, so remove them.  */
   {
-    int writei;
-    int readi;
     int nsyms_old = nsyms;
-    for (writei = 0, readi = 0; readi < nsyms_old; readi += 1)
+    for (int writei = 0, readi = 0; readi < nsyms_old; readi += 1)
       {
         if (symbols[readi] == NULL)
           {
@@ -998,9 +1024,7 @@ symbols_pack (void)
         else
           {
             symbols[writei] = symbols[readi];
-            symbols[writei]->number = writei;
-            if (symbols[writei]->alias)
-              symbols[writei]->alias->number = writei;
+            symbols[writei]->content->number = writei;
             writei += 1;
           }
       }
@@ -1009,11 +1033,11 @@ symbols_pack (void)
 
   symbols_token_translations_init ();
 
-  if (startsymbol->class == unknown_sym)
+  if (startsymbol->content->class == unknown_sym)
     complain (&startsymbol_location, fatal,
               _("the start symbol %s is undefined"),
               startsymbol->tag);
-  else if (startsymbol->class == token_sym)
+  else if (startsymbol->content->class == token_sym)
     complain (&startsymbol_location, fatal,
               _("the start symbol %s is a token"),
               startsymbol->tag);
@@ -1026,9 +1050,8 @@ symbols_pack (void)
 static void
 init_prec_nodes (void)
 {
-  int i;
   prec_nodes = xcalloc (nsyms, sizeof *prec_nodes);
-  for (i = 0; i < nsyms; ++i)
+  for (int i = 0; i < nsyms; ++i)
     {
       prec_nodes[i] = xmalloc (sizeof *prec_nodes[i]);
       symgraph *s = prec_nodes[i];
@@ -1119,8 +1142,7 @@ linkedlist_free (symgraphlink *node)
 static void
 assoc_free (void)
 {
-  int i;
-  for (i = 0; i < nsyms; ++i)
+  for (int i = 0; i < nsyms; ++i)
     {
       linkedlist_free (prec_nodes[i]->pred);
       linkedlist_free (prec_nodes[i]->succ);
@@ -1136,9 +1158,8 @@ assoc_free (void)
 static void
 init_assoc (void)
 {
-  graphid i;
   used_assoc = xcalloc (nsyms, sizeof *used_assoc);
-  for (i = 0; i < nsyms; ++i)
+  for (graphid i = 0; i < nsyms; ++i)
     used_assoc[i] = false;
 }
 
@@ -1150,9 +1171,9 @@ static inline bool
 is_assoc_useless (symbol *s)
 {
   return s
-      && s->assoc != undef_assoc
-      && s->assoc != precedence_assoc
-      && !used_assoc[s->number];
+      && s->content->assoc != undef_assoc
+      && s->content->assoc != precedence_assoc
+      && !used_assoc[s->content->number];
 }
 
 /*-------------------------------.
@@ -1175,28 +1196,27 @@ register_assoc (graphid i, graphid j)
 void
 print_precedence_warnings (void)
 {
-  int i;
   if (!prec_nodes)
     init_prec_nodes ();
   if (!used_assoc)
     init_assoc ();
-  for (i = 0; i < nsyms; ++i)
+  for (int i = 0; i < nsyms; ++i)
     {
       symbol *s = symbols[i];
       if (s
-          && s->prec != 0
+          && s->content->prec != 0
           && !prec_nodes[i]->pred
           && !prec_nodes[i]->succ)
         {
           if (is_assoc_useless (s))
-            complain (&s->prec_location, Wprecedence,
+            complain (&s->content->prec_location, Wprecedence,
                       _("useless precedence and associativity for %s"), s->tag);
-          else if (s->assoc == precedence_assoc)
-            complain (&s->prec_location, Wprecedence,
+          else if (s->content->assoc == precedence_assoc)
+            complain (&s->content->prec_location, Wprecedence,
                       _("useless precedence for %s"), s->tag);
         }
       else if (is_assoc_useless (s))
-        complain (&s->prec_location, Wprecedence,
+        complain (&s->content->prec_location, Wprecedence,
                   _("useless associativity for %s, use %%precedence"), s->tag);
     }
   free (used_assoc);
